@@ -24,22 +24,18 @@ Data::Dumper
 
 Storable
 
+URI::Escape
+
+CGI::Cookie
+
 Bundle::LWP
 
 Net::SMTP
 
 Mail::Internet 1.32
 
-ExtUtils::PerlPP
-
-Bundle::DBD::CSV
-
 HTML::EP
 
-URI::Escape
-
-CGI::Cookie
-
 =head1 DESCRIPTION
 
 This bundle includes all that's needed to run the HTML::EP package with
